pub struct XCMSender<Sender, Destination, Message, ToXcm>(/* private fields */);Expand description
Common utility to send XCM messages that can use SplittableMessage.
It can be used both in the RC and AH. Message is the splittable message type, and ToXcm
should be configured by the user, converting message to a valida Xcm<()>. It should utilize
the correct call indices, which we only know at the runtime level.
Implementations
Sourceimpl<Sender, Destination, Message, ToXcm> XCMSender<Sender, Destination, Message, ToXcm>
impl<Sender, Destination, Message, ToXcm> XCMSender<Sender, Destination, Message, ToXcm>
Sourcepub fn send(message: Message) -> Result<(), ()>
pub fn send(message: Message) -> Result<(), ()>
Send the message single-shot; no splitting.
Useful for sending messages that are already paged/chunked, so we are sure that they fit in one message.
Sourcepub fn send_with_fees<XcmExec, Call, AccountId, AccountToLoc, Balance>(
message: Message,
payer: AccountId,
max_delivery_and_remote_execution_fee: Option<Balance>,
execution_cost: Balance,
) -> Result<Balance, SendKeysError<Balance>>
pub fn send_with_fees<XcmExec, Call, AccountId, AccountToLoc, Balance>( message: Message, payer: AccountId, max_delivery_and_remote_execution_fee: Option<Balance>, execution_cost: Balance, ) -> Result<Balance, SendKeysError<Balance>>
Send the message with fee charging and optional max fee limit.
This method validates the XCM message first, calculates the total fee (delivery + execution), optionally checks if the total exceeds the specified maximum, charges the total from the payer, and then delivers the message.
The relay chain uses UnpaidExecution, so no fees are charged there. Instead, the
total cost (delivery + remote execution) is charged upfront on AssetHub.
message: The message to sendpayer: The account paying feesmax_delivery_and_remote_execution_fee: Optional maximum total fee the user is willing to payexecution_cost: The relay chain execution cost to include in the total
Generic parameters:
XcmExec: The XCM executor that implementscharge_feesCall: The runtime call type (used by XcmExec)AccountId: The account identifier typeAccountToLoc: Converter from AccountId to XCM LocationBalance: The balance type for fee limits
Returns the total fees charged on success (delivery + execution).
Sourceimpl<Sender, Destination, Message, ToXcm> XCMSender<Sender, Destination, Message, ToXcm>where
Sender: SendXcm,
Destination: Get<Location>,
Message: SplittableMessage + Display + Clone + Encode,
ToXcm: Convert<Message, Xcm<()>>,
impl<Sender, Destination, Message, ToXcm> XCMSender<Sender, Destination, Message, ToXcm>where
Sender: SendXcm,
Destination: Get<Location>,
Message: SplittableMessage + Display + Clone + Encode,
ToXcm: Convert<Message, Xcm<()>>,
Sourcepub fn split_then_send(
message: Message,
maybe_max_steps: Option<u32>,
) -> Result<(), ()>
👎Deprecated: all staking related VMP messages should fit the single message limits. Should not be used.
pub fn split_then_send( message: Message, maybe_max_steps: Option<u32>, ) -> Result<(), ()>
Safe send method to send a message, while validating it and using SplittableMessage to
split it into smaller pieces if XCM validation fails with ExceedsMaxMessageSize. It will
fail on other errors.
Returns Ok() if the message was sent using XCM, potentially with splitting up to
maybe_max_step times, Err(()) otherwise.